/********************************************************************
Credit: AVINASH GUPTA (PulseIn from Arduino into C Code)
********************************************************************/

#include <avr/io.h>
#include <util/delay.h>
#include "peri2.h"

#define US_PORT PORTC
#define	US_PIN	PINC
#define US_DDR 	DDRC

#define US_TRIG1_POS	PC0
#define US_ECHO1_POS	PC1
#define US_TRIG2_POS	PC2
#define US_ECHO2_POS	PC3

#define US_ERROR	-1
#define	US_NO_OBSTACLE	-2

void SonicInit();
void SonicTrigger();

void SonicInit()
{
	US_DDR|=(1<<US_TRIG1_POS);
	US_DDR|=(1<<US_TRIG2_POS);
}

void SonicTrigger()
{
	//Send a 10uS pulse on trigger line
	
	US_PORT|=(1<<US_TRIG1_POS);	//high
	US_PORT|=(1<<US_TRIG2_POS);
	_delay_us(15);			//wait 15uS
	
	US_PORT&=~(1<<US_TRIG1_POS);	//low
	US_PORT&=~(1<<US_TRIG2_POS);
}

uint16_t GetPulseWidth(uint8_t US_ECHO_POS)
{
	uint32_t i,result;

	//Wait for the rising edge
	for(i=0;i<600000;i++)
	{
		if(!(US_PIN & (1<<US_ECHO_POS))) 
			continue;	//Line is still low, so wait
		else 
			break;		//High edge detected, so break.
	}

	if(i==600000)
		return US_ERROR;	//Indicates time out
	
	//High Edge Found

	//Setup Timer1
	TCCR1A=0X00;
	TCCR1B=(1<<CS11);		//Prescaler = Fcpu/8
	TCNT1=0x00;			//Init counter

	//Now wait for the falling edge
	for(i=0;i<600000;i++)
	{
		if(US_PIN & (1<<US_ECHO_POS))
		{
			if(TCNT1 > 60000) break; else continue;
		}
		else
			break;
	}

	if(i==600000)
		return US_NO_OBSTACLE;	//Indicates time out

	//Falling edge found

	result=TCNT1;

	//Stop Timer
	TCCR1B=0x00;

	if(result > 60000)
		return US_NO_OBSTACLE;	//No obstacle
	else
		return (result>>1);
}

int main()
{
	uint16_t r1,r2;
	int chair, table;
	
	init_peripheral();

	SonicInit();			//Set port direction of sensor
	
	while(1)
	{
		//Send a trigger pulse
		SonicTrigger();

		//Measure the width of pulse
		r1 = GetPulseWidth(US_ECHO2_POS);
		r2 = GetPulseWidth(US_ECHO1_POS);

		//Handle Errors
		if(r1==US_ERROR && r2==US_ERROR)
		{
			//return 0;
		}
		else if(r1==US_NO_OBSTACLE && r2==US_NO_OBSTACLE)
		{
			//return 0;
		}
		else
		{
			int d1,d2;

			d1=(r1/58.0);	//Convert to cm
			d2=(r2/58.0);

			chair = Check(d1);
			table = Check(d2);

			if(chair == table)
			{
				if(chair == 0)
				{
					set_led_value(0b001);
				}
				else
				{
					set_led_value(0b100000);
				}
			}
			else
			{
				set_led_value(0b010);
			}
			_delay_ms(500);
		}
	}
}

int Check(int d)
{
	if(d>=0 && d<15)
	{
		return 0;
	}
	else
	{
		return 1;
	}
}
